/*
 * AFE4490.c
 *
 *  Created on: May 4, 2018
 *      Author: clairegaffney
 */

#include <rsl10.h>

/* Defines */
#define CONCAT(x, y)                    x##y //concatenation operator
#define SPI0_WORD_SIZE(x)               CONCAT(SPI0_WORD_SIZE_, x) //defining word size
#define DIO_SRC(x)                      CONCAT(DIO_SRC_DIO_, x) //selecting DIO as source
#define SPI_TX_DATAr                    0
#define TX_BUFFER_SIZE                  sizeof(SPI_TX_DATAr) //defining the size of the transmit buffer
#define RX_BUFFER_SIZE                  (2 * TX_BUFFER_SIZE) //size of data reception buffer should be 2 times that of transmission
#define SPI_CLK_DIO                     8 // SPI clock
#define SPI_CS_DIO                      11 //chip select (SPISTE)
#define SPI_SERI_DIO                    10 //SIMO
#define SPI_SERO_DIO                    9 //SOMI
#define BUTTON_DIO                      4 // start/stop data transfer button
#define WORD_SIZE                       24 //need 24 byte data

/* DIO number that is used for easy re-flashing (recovery mode) */
#define RECOVERY_DIO                    12

/* Variable declarations and initializations */
volatile uint32_t SPI_TX_DATA = 0;

volatile uint8_t spi_start       = 0;
volatile uint8_t spi_finish      = 0;
volatile uint8_t nbr_tx_send     = 0;
volatile uint32_t tx_buffer[TX_BUFFER_SIZE] = {0};
volatile uint8_t tx_buffer_index = 0;
volatile uint8_t nbr_rx_received = 0;
volatile uint8_t rx_buffer_index = 0;
volatile uint8_t rx_buffer[RX_BUFFER_SIZE];

/* Function declarations */
extern void DIO0_IRQHandler(void);
extern void SPI0_TX_IRQHandler(void);
extern void SPI0_RX_IRQHandler(void);
void Initialize(void);
void LED_flashing(void);

/* Start SPI transfer */
void DIO0_IRQHandler(void)
{
    static uint8_t ignore_next_dio_int = 0;
    if (ignore_next_dio_int)
    {
        ignore_next_dio_int = 0;
    }
    else if (DIO_DATA->ALIAS[BUTTON_DIO] == 0)
    {
        /* Button is pressed: Ignore next interrupt.
         * This is required to deal with the debounce circuit limitations. */
        ignore_next_dio_int = 1;

        /* Set the spi_start status */
        spi_start = 1;
    }
}

/* Send the next byte, when the transfer is finished put SPI in IDLE */
void SPI0_TX_IRQHandler(void)
{
    nbr_tx_send++;
    if (tx_buffer_index < TX_BUFFER_SIZE)
    {
        SPI0->TX_DATA = tx_buffer[tx_buffer_index++];
        Sys_SPI_ReadWrite(0, WORD_SIZE);
    }
    else
    {
        spi_finish = 1;
        Sys_SPI_TransferConfig(0, SPI0_IDLE | SPI0_WRITE_DATA |
                               SPI0_CS_1 | SPI0_WORD_SIZE(WORD_SIZE));
    }
}

/* Receive data and put it in a circular buffer */
void SPI0_RX_IRQHandler(void)
{
    nbr_rx_received++;
    if (rx_buffer_index >= RX_BUFFER_SIZE)
    {
        rx_buffer_index = 0;
    }
    rx_buffer[rx_buffer_index++] = SPI0->RX_DATA;
}

/*Initialize the system by disabling interrupts,
* switching to an internal clock, configuring the required DIOs for an SPI interface
*  and enabling the interrupts
*/
void Initialize(void)
{
    /* Mask all interrupts */
    __set_PRIMASK(PRIMASK_DISABLE_INTERRUPTS);

    /* Disable all existing interrupts, clearing all pending source */
    Sys_NVIC_DisableAllInt();
    Sys_NVIC_ClearAllPendingInt();



    /*in the example program, there's a test to 'pause' the program and also something about a
    	 * debounce filter but i didn't include it here because it didn't seem necessary but i'm adding a note
    	 * because lord knows it could be essential*/

    /* Configure SPI0 as Master */
    Sys_SPI_Config(0, SPI0_OVERRUN_INT_DISABLE  | SPI0_UNDERRUN_INT_DISABLE |
                   SPI0_CONTROLLER_CM3       | SPI0_SELECT_MASTER        |
                   SPI0_CLK_POLARITY_NORMAL  | SPI0_MODE_SELECT_MANUAL   |
                   SPI0_ENABLE               | SPI0_PRESCALE_4);

    /* Configure the type of transfer for SPI 0 */
    Sys_SPI_TransferConfig(0, SPI0_IDLE | SPI0_WRITE_DATA |
                           SPI0_CS_1 |  SPI0_WORD_SIZE(WORD_SIZE));

    /*Configure the DIOs for SPI0 as Master Mode
    	 * Sys_SPI_DIOConfig(num, slave, cfg, clk, cs, serio, sero)
    	 * num   - interface (0/1)                         = 0
    	 * slave - master/slave configuration              = master
    	 * cfg   - DIO pin configuration for the SPI pads  = 11 --i'm not really sure why you wouldn't just put the DIO number (maybe the example assumed you would define it earlier?)
    	 * clk   - DIO to use as the SPI clock pad         = 8
    	 * cs 	 - DIO to use as the chip select pad       = 1
    	 * seri  - DIO to use as the SPI serial input pad  = (SPISIMO?) 10
    	 * sero  - DIO to use as the SPI serial output pad = (SPISOMI?) 9
    	 *
    	 * */
    Sys_SPI_DIOConfig(0, SPI0_SELECT_MASTER,
                      DIO_6X_DRIVE | DIO_LPF_DISABLE | DIO_WEAK_PULL_UP,
                      SPI_CLK_DIO, SPI_CS_DIO, SPI_SERI_DIO, SPI_SERO_DIO);

    /* Enable interrupts */
    NVIC_EnableIRQ(DIO0_IRQn);
    NVIC_EnableIRQ(SPI0_RX_IRQn);
    NVIC_EnableIRQ(SPI0_TX_IRQn);

    /* Stop masking interrupts */
    __set_PRIMASK(PRIMASK_ENABLE_INTERRUPTS);
    __set_FAULTMASK(FAULTMASK_ENABLE_INTERRUPTS);
}

void LED_flashing(void)
{
	// LED2STC
	SPI_TX_DATA = 0x0017A2;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED2ENDC
	SPI_TX_DATA = 0x001F3E;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED2LEDSTC
	SPI_TX_DATA = 0x001770;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED2LEDENDC
	SPI_TX_DATA = 0x001F3F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED2STC
	SPI_TX_DATA = 0x000050;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED2ENDC
	SPI_TX_DATA = 0x0007CE;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1STC
	SPI_TX_DATA = 0x000820;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1ENDC
	SPI_TX_DATA = 0x000F9E;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1LEDSTC
	SPI_TX_DATA = 0x007D0;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1LEDENDC
	SPI_TX_DATA = 0x000F9F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED1STC
	SPI_TX_DATA = 0x000FF0;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED1ENDC
	SPI_TX_DATA = 0x00176E;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED2CONVST
	SPI_TX_DATA == 0x000006;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED2CONVEND
	SPI_TX_DATA == 0x0007CF;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED2CONVST
	SPI_TX_DATA = 0x0007D6;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED2CONVEND
	SPI_TX_DATA = 0x000F9F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1CONVST
	SPI_TX_DATA = 0x000FA6;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//LED1CONVEND
	SPI_TX_DATA = 0x00176F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED1CONVST
	SPI_TX_DATA = 0x001776;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ALED1CONVEND
	SPI_TX_DATA = 0x001F3F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTSTCT0
	SPI_TX_DATA = 0x000000;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTENDCT0
	SPI_TX_DATA = 0x000005;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTSTCT1
	SPI_TX_DATA = 0x0007D0;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTENDCT1
	SPI_TX_DATA = 0x0007D5;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTSTCT2
	SPI_TX_DATA = 0x000FA0;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTENDCT2
	SPI_TX_DATA = 0x000FA5;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTSTCT3
	SPI_TX_DATA = 0x001770;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//ADCRSTENDCT3
	SPI_TX_DATA = 0x001775;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/

	//PRPCOUNT
	SPI_TX_DATA = 0x001F3F;
	SPI0->TX_DATA = tx_buffer[tx_buffer_index++]; /*put the new value on spi tx buffer*/
	Sys_SPI_ReadWrite(0, WORD_SIZE); /*start the send of the frame*/
}

/*
 *  Initialize the system, then send an SPI frame controlled
 *                 by DIO5 (press to send). */
int main(void)
{
    /* Initialize the system */
    Initialize();

    /* Spin loop */
    while (1)
    {
        /* Refresh the watchdog timer */
        Sys_Watchdog_Refresh();
        if (spi_start == 1)
        {
            spi_start = 0;

            /* Lower CS to give time for the slave to prepare */
            Sys_SPI_TransferConfig(0, SPI0_IDLE | SPI0_WRITE_DATA |
                                   SPI0_CS_0 | SPI0_WORD_SIZE(WORD_SIZE));

            /* Restart indexes */
            tx_buffer_index = 0;

            /* Put the new value on spi tx buffer */
            SPI0->TX_DATA   = tx_buffer[tx_buffer_index++];

            /* Start the send of the frame */
            Sys_SPI_ReadWrite(0, WORD_SIZE);

            /* A transmission started, set the LED for 0.5 s */
            Sys_GPIO_Set_High(LED_DIO);
            Sys_Delay_ProgramROM(0.5 * SystemCoreClock);
            Sys_GPIO_Set_Low(LED_DIO);
        }

        /* SPI transaction finished */
        if (spi_finish == 1)
        {
            /* Check the number of byte sent and received.
             * Check the data received is as expected. If data is correct
             * set the LED for 0.5 s */
            if (nbr_tx_send == nbr_rx_received)
            {
                if (rx_buffer[rx_buffer_index - 1] == nbr_tx_send - 1)
                {
                    /* Wait 0.5 s before setting the led so the user can see
                     * the blink */
                    Sys_Delay_ProgramROM(0.5 * SystemCoreClock);
                    Sys_GPIO_Set_High(LED_DIO);
                    Sys_Delay_ProgramROM(0.5 * SystemCoreClock);
                    Sys_GPIO_Set_Low(LED_DIO);
                }
            }
            spi_finish = 0;
        }
    }
}

